@@ -311,7 +311,7 @@ class BoundingBox(object):
                 isinstance(self._ymax, int))
                 
     def int(self):
-        """Convert corners to integer with rounding"""
+        """Convert corners to integer with rounding, in-place update"""
         (w,h) = (int(np.round(self.bbwidth())), int(np.round(self.bbheight())))
         self._xmin = int(np.round(self._xmin))
         self._ymin = int(np.round(self._ymin))
@@ -513,7 +513,7 @@ class BoundingBox(object):
         assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))
         return np.exp(-self.sqdist(bb)/(float(2*self.maxdim()*self.maxdim()) if sigma is None else float(2.0*sigma*sigma)))
 
-    def iou(self, bb):
+    def iou(self, bb, area=None, otherarea=None):
         """area of intersection / area of union"""
         assert bb is None or isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))        
         if bb is None:
@@ -526,16 +526,19 @@ class BoundingBox(object):
             return 0  # invalid (no overlap), early exit
 
         area_intersection = w * h
-        area_union = (self.area() + bb.area() - area_intersection)
+        area_union = ((self.area() if area is None else area) +
+                      (bb.area() if otherarea is None else otherarea) -
+                      area_intersection)
         return area_intersection / float(area_union)
 
     def intersection_over_union(self, bb):
         """Alias for iou"""
         return self.iou(bb)
 
-    def area_of_intersection(self, bb):
+    def area_of_intersection(self, bb, strict=True):
         """area of intersection"""
-        assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))                
+        if strict:
+            assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))                
         w = min(self._xmax, bb._xmax) - max(self._xmin, bb._xmin)
         if w <= 0:
             return 0  # invalid (no overlap), early exit 
@@ -544,22 +547,28 @@ class BoundingBox(object):
             return 0  # invalid (no overlap), early exit 
         return w*h
 
+    def area_of_union(self, bb):
+        return self.area() + bb.area() - self.area_of_intersection(bb)
+        
     def cover(self, bb):
         """Fraction of this bounding box intersected by other bbox (bb)"""        
         return self.area_of_intersection(bb) / float(self.area())
 
-    def maxcover(self, bb):
+    def maxcover(self, bb, area=None, otherarea=None):
         """The maximum cover of self to bb and bb to self"""
-        return max(self.cover(bb), bb.cover(self))
+        aoi = self.area_of_intersection(bb, strict=False)
+        return float(max(aoi/(self.area() if area is None else area), aoi/(bb.area() if otherarea is None else otherarea)))
     
-    def shapeiou(self, bb):
+    def shapeiou(self, bb, area=None, otherarea=None):
         """Shape IoU is the IoU with the upper left corners aligned. This measures the deformation of the two boxes by removing the effect of translation"""
         #return self.iou(bb.clone().translate(dx=self._xmin-bb._xmin, dy=self._ymin-bb._ymin))  # equivalent to
         assert isinstance(bb, BoundingBox), "Invalid input - must be BoundingBox()"
         w = min(self._xmax, bb._xmax + (self._xmin-bb._xmin)) - max(self._xmin, bb._xmin + (self._xmin-bb._xmin))
         h = min(self._ymax, bb._ymax + (self._ymin-bb._ymin)) - max(self._ymin, bb._ymin + (self._ymin-bb._ymin))
         area_intersection = w * h
-        area_union = (self.area() + bb.area() - area_intersection)
+        area_union = ((self.area() if area is None else area) +
+                      (bb.area() if otherarea is None else otherarea)
+                      - area_intersection)
         return area_intersection / float(area_union)
         
     def intersection(self, bb, strict=True):
@@ -573,11 +582,23 @@ class BoundingBox(object):
             raise ValueError('Degenerate intersection for bounding boxes "%s" and "%s"' % (str(bb), str(self)))
         return self
 
-    def hasintersection(self, bb):
-        """Return true of self and bb intersect, do not perform type check to maximize speed"""
-        #assert isinstance(bb, BoundingBox), "Invalid BoundingBox() input of type '%s'" % str(type(bb))
-        return (((self._xmax if self._xmax < bb._xmax else bb._xmax) - (self._xmin if self._xmin > bb._xmin else bb._xmin)) > 0 and
-                ((self._ymax if self._ymax < bb._ymax else bb._ymax) - (self._ymin if self._ymin > bb._ymin else bb._ymin)) > 0)
+    def hasintersection(self, bb, iou=None, cover=None, maxcover=None, bbcover=None, area=None, otherarea=None, gate=0):
+        """Return true if self and bb overlap by any amount, or by the cover threshold (if provided) or the iou threshold (if provided).  This is a convenience function that allows for shared computation for fast non-maximum suppression."""
+
+        if not (((self._xmax if self._xmax < bb._xmax else bb._xmax) - (self._xmin if self._xmin > bb._xmin else bb._xmin)) > (-gate) and
+                ((self._ymax if self._ymax < bb._ymax else bb._ymax) - (self._ymin if self._ymin > bb._ymin else bb._ymin)) > (-gate)):  # faster than min(x,y)-max(x,y)
+            return False  # does not intersect
+        
+        elif maxcover is not None or iou is not None or cover is not None or bbcover is not None:
+            aoi = self.area_of_intersection(bb, strict=False)            
+            otherarea = otherarea if otherarea is not None else (bb.area() if (maxcover is not None or bbcover is not None or iou is not None) else 0)
+            area = area if area is not None else (self.area() if (maxcover is not None or cover is not None or iou is not None) else 0)
+            return (((maxcover is not None) and (max(aoi/area, aoi/otherarea) > maxcover)) or
+                    ((iou is not None) and ((aoi / (area+otherarea-aoi)) >= iou)) or
+                    ((cover is not None) and ((aoi / area) >= cover)) or
+                    ((bbcover is not None) and ((aoi / otherarea) >= bbcover)))
+        else:
+            return True
 
     def union(self, bb):
         """Union of one or more bounding boxes with this box"""        
@@ -908,6 +929,11 @@ class BoundingBox(object):
         """Bounding boxes do not have confidences, use vipy.object.Detection()"""
         return None
 
+    def grid(self, rows, cols):
+        """Split a bounding box into the smallest grid of non-overlapping bounding boxes such that the union is the original box"""
+        (w,h) = (self.width()/cols, self.height()/rows)
+        return [BoundingBox(xmin=x, ymin=y, width=w, height=h) for x in np.arange(self._xmin, self._xmax, w) for y in np.arange(self._ymin, self._ymax, h)]
+
 class Ellipse():
     __slots__ = ['_major', '_minor', '_xcenter', '_ycenter', '_phi']
     def __init__(self, semi_major, semi_minor, xcenter, ycenter, phi):
@@ -979,3 +1005,7 @@ class Ellipse():
         for (y,x) in product(range(0,H), range(0,W)):
             img[y,x] = self.inside(x,y)
         return img
+
+def union(bblist):
+    """Return the union of a list of vipy.geometry.BoundingBox"""
+    return bblist[0].clone().union(bblist)    
