@@ -4,6 +4,7 @@ from vipy.util import isstring, tolist, chunklistwithoverlap, try_import, Timer
 import uuid
 import copy
 import warnings
+from itertools import islice
 
 try:
     import ujson as json  # faster
@@ -226,12 +227,12 @@ class Track(object):
 
     def __getitem__(self, k):
         """Interpolate the track at frame k"""
-        return self._linear_interpolation(k)
+        return self.linear_interpolation(k)
 
     def __iter__(self):
         """Iterate over the track interpolating each frame from min(keyframes) to max(keyframes)"""
         for k in range(self.startframe(), self.endframe()+1):
-            yield self._linear_interpolation(k)
+            yield self.linear_interpolation(k)
 
     def __len__(self):
         """The length of a track is the total number of interpolated frames, or zero if degenerate"""
@@ -245,6 +246,8 @@ class Track(object):
         if samples is not None:
             dt = max(1, int(round(len(self._keyframes)/float(samples))))
             C = [self._keyboxes[i]._confidence for i in range(len(self._keyframes)-1, 0, -dt) if (hasattr(self._keyboxes[i], '_confidence') and self._keyboxes[i]._confidence is not None)]
+        elif last == 1:
+            return self.endbox().confidence() if len(self)>0 else 0
         else:
             ef = self.endframe() - last if last is not None else 0
             C = [d._confidence for (f,d) in zip(self.keyframes(), self.keyboxes()) if f >= ef and (hasattr(d, '_confidence') and d._confidence is not None)]
@@ -273,6 +276,7 @@ class Track(object):
         self._keyframes.append(int(keyframe))
         self._keyboxes.append(bbox)
         if len(self._keyframes) > 1 and keyframe < self._keyframes[-2]:
+            # Preserve sorted order if inserting into the middle somewhere
             (self._keyframes, self._keyboxes) = zip(*sorted([(f,bb) for (f,bb) in zip(self._keyframes, self._keyboxes)], key=lambda x: x[0]))        
             self._keyframes = list(self._keyframes)
             self._keyboxes = list(self._keyboxes)
@@ -347,7 +351,7 @@ class Track(object):
     def endframe(self):
         return self._keyframes[-1] if len(self._keyframes)>0 else None  # assumes sorted order
 
-    def _linear_interpolation(self, f):
+    def linear_interpolation(self, f, id=True):
         """Linear bounding box interpolation at frame=k given observed boxes (x,y,w,h) at keyframes.  
         This returns a vipy.object.Detection() which is the interpolation of the Track() at frame k
         If self._boundary='extend', then boxes are repeated if the interpolation is outside the keyframes
@@ -375,7 +379,8 @@ class Track(object):
                       ymax=bi._ymax + c*(bj._ymax - bi._ymax),   # float(np.interp(k, self._keyframes, [bb._ymax for bb in self._keyboxes])),
                       confidence=bi.confidence(),  # may be None
                       category=self.category(),
-                      shortlabel=self.shortlabel())
+                      shortlabel=self.shortlabel(),
+                      id=id)
         d.attributes['trackid'] = self.id()  # for correspondence of detections to tracks
         return d if self._boundary == 'extend' or self.during(f) else None
 
@@ -402,8 +407,12 @@ class Track(object):
     def during(self, k_start, k_end=None):
         """Is frame during the time interval (startframe, endframe) inclusive?"""        
         k_end = k_start+1 if k_end is None else k_end
-        #return len(self)>0 and any([k >= self.startframe() and k <= self.endframe() for k in range(k_start, k_end)])
-        return len(self)>0 and ((k_start >= self._keyframes[0] and k_start <= self._keyframes[-1]) or (k_end >= self._keyframes[0] and k_end <= self._keyframes[-1]))
+        (startframe, endframe) = (self.startframe(), self.endframe())
+        if len(self)>0:
+            for k in range(k_start, k_end):
+                if k >= startframe and k <= endframe:
+                    return True  # early exit
+        return False
 
     def during_interval(self, k_start, k_end):
         return self.during(k_start, k_end)
@@ -420,11 +429,8 @@ class Track(object):
         return self
 
     def truncate(self, startframe=None, endframe=None):
-        """Truncate a track so that any keyframes less than startframe or greater than or equal to endframe are removed"""
-        keyframes = copy.deepcopy(self.keyframes())
-        for k in keyframes:
-            if ((startframe is not None and k < startframe) or (endframe is not None and k >= endframe)):
-                self.delete(k)  # will also delete corresponding keybox
+        """Truncate a track so that any keyframes less than startframe or greater than endframe are removed"""
+        (self._keyframes, self._keyboxes) = zip(*[(kf,kb) for (kf,kb) in zip(self._keyframes, self._keyboxes) if ((startframe is None or kf >= startframe) and (endframe is None or kf <= endframe))])
         return self
         
     def rescale(self, s):
@@ -732,12 +738,9 @@ class Track(object):
             d = self.endbox().clone() if k > self.endframe() else self.startbox().clone()
             (vx, vy) = self.shape_invariant_velocity(n, dt=dt) if not shape else self.velocity(n, dt=dt)
             (vw, vh) = (self.velocity_w(n, dt=dt), self.velocity_h(n, dt=dt)) if shape else (0,0)
-            return (d.translate((k-n)*vx, (k-n)*vy)
-                    .top(0 if not shape else ((k-n)*vh)/2.0)
-                    .bottom(0 if not shape else ((k-n)*vh)/2.0)
-                    .left(0 if not shape else ((k-n)*vw)/2.0)
-                    .right(0 if not shape else ((k-n)*vw)/2.0))
-       
+            d = d.translate((k-n)*vx, (k-n)*vy)
+            return d if not shape else d.top( ((k-n)*vh)/2.0).bottom( ((k-n)*vh)/2.0).left( ((k-n)*vw)/2.0).right( ((k-n)*vw)/2.0)
+            
     def imclip(self, width, height):
         """Clip the track to the image rectangle (width, height).  If a keybox is outside the image rectangle, remove it otherwise clip to the image rectangle. 
            This operation can change the length of the track and the size of the keyboxes.  The result may be an empty track if the track is completely outside
@@ -797,19 +800,15 @@ class Track(object):
         return self
 
     def shape_invariant_velocity(self, f, dt=30):
-        """Return the (x,y) track velocity at frame f in units of pixels per frame computed by minimum mean finite differences of any box corner independent of changes in shape"""
-        assert f >= 0 and dt > 0 
-        bbl = [(max(1,(f-self._keyframes[k])), bb) for (k,bb) in enumerate(self._keyboxes) if self._keyframes[k] >= f-dt]
-        vx = float(np.mean([sorted([(bbl[-1][1].ulx() - bb.ulx())/float(k), 
-                                    (bbl[-1][1].urx() - bb.urx())/float(k), 
-                                    (bbl[-1][1].blx() - bb.blx())/float(k), 
-                                    (bbl[-1][1].brx() - bb.brx())/float(k)], key=lambda x: abs(x))[0]
-                            for (k,bb) in bbl[0:-1]])) if (self.during(f-1) and self.during(f)) else 0
-        vy = float(np.mean([sorted([(bbl[-1][1].uly() - bb.uly())/float(k), 
-                                    (bbl[-1][1].ury() - bb.ury())/float(k), 
-                                    (bbl[-1][1].bly() - bb.bly())/float(k), 
-                                    (bbl[-1][1].bry() - bb.bry())/float(k)], key=lambda x: abs(x))[0]
-                            for (k,bb) in bbl[0:-1]])) if (self.during(f-1) and self.during(f)) else 0
+        """Return the (x,y) track velocity at frame f in units of pixels per frame computed by minimum mean finite differences of any box corner independent of changes in shape, over a finite time window of [f-dt, f]"""
+        assert f >= 0 and dt > 0
+        if len(self) < 2 or not (self.during(f) and self.during(f-dt)) :
+            return (0,0)
+        
+        kb = [((f-dt), self.linear_interpolation(f-dt, id=False))] + [(kf, bb) for (kf,bb) in zip(self._keyframes, self._keyboxes) if (kf > f-dt) and (kf < f)]
+        (kfe, bbe) = (f, self.linear_interpolation(f, id=False))
+        vx = float((1.0/len(kb))*sum([min([(bbe._xmin - bb._xmin), (bbe._xmax - bb._xmax)], key=abs)/float(kfe-kf) for (kf,bb) in kb]))
+        vy = float((1.0/len(kb))*sum([min([(bbe._ymin - bb._ymin), (bbe._ymax - bb._ymax)], key=abs)/float(kfe-kf) for (kf,bb) in kb]))
         return (vx, vy)
 
     def velocity_x(self, f, dt=30):
@@ -847,25 +846,37 @@ class Track(object):
         (bbs, bbe) = (self[max(self.startframe(), startframe)] if startframe is not None else self.startbox(), self[min(self.endframe(), endframe)] if endframe is not None else self.endbox())
         return (bbs.maxcover(bbe) < mincover) if (bbs is not None and bbe is not None) else False
 
-
-def non_maximum_suppression(detlist, conf, iou, bycategory=False, cover=None, coverdilation=1.2):
+    
+def non_maximum_suppression(detlist, conf, iou, bycategory=False, cover=None, gridsize=(6,9)):
     """Compute greedy non-maximum suppression of a list of vipy.object.Detection() based on spatial IOU threshold (iou) and cover threhsold (cover) sorted by confidence (conf)"""
     assert all([isinstance(d, Detection) for d in detlist])
     assert all([d.confidence() is not None for d in detlist])
     assert conf>=0 and iou>=0 and iou<=1
     assert cover is None or (cover>=0 and cover<=1)
-
+    assert isinstance(gridsize, tuple) and len(gridsize) == 2
+        
     suppressed = set([])
     detlist = [d for d in detlist if d.confidence() > conf and not d.isdegenerate()]  # valid
-    detlist = sorted(detlist, key=lambda d: d.confidence(), reverse=True)  # biggest to smallest
+    detlist.sort(key=lambda d: d.confidence(), reverse=True)  # biggest to smallest, in-place
+    grid = detlist[0].clone().union(detlist).grid(gridsize[0], gridsize[1]) if len(detlist) > 0 else []
+    bbidx = [set([k for (k,bbg) in enumerate(grid) if (((bbg._xmax if bbg._xmax < bb._xmax else bb._xmax) - (bbg._xmin if bbg._xmin > bb._xmin else bb._xmin)) > 0 and
+                                                       ((bbg._ymax if bbg._ymax < bb._ymax else bb._ymax) - (bbg._ymin if bbg._ymin > bb._ymin else bb._ymin)) > 0)])
+             for bb in detlist]  # spatial index, without the function call overhead of bbg.hasintersection(bb)
+    #bbidx = [set([k for (k,bbg) in enumerate(grid) if bbg.hasintersection(bb)]) for bb in detlist]  # spatial index, equivalent to above but slower
+    
+    area = [bb.area() for bb in detlist]
     for (i, di) in enumerate(detlist):
         if i in suppressed:
             continue
-        ddi = di.clone().dilate(coverdilation)
-        for (j, dj) in enumerate(detlist[i+1:], start=i+1):
-            if (j not in suppressed) and (bycategory is False or di._label == dj._label) and ddi.hasintersection(dj) and ((di.iou(dj) >= iou) or (cover is not None and ddi.cover(dj) >= cover)):
+        for (j, dj) in enumerate(islice(detlist, i+1, None), start=i+1):  # no-copy, equivalent to detlist[i+1:]
+            if ((j not in suppressed) and
+                (bycategory is False or di._label == dj._label) and
+                (not bbidx[i].isdisjoint(bbidx[j])) and
+                ((cover is not None and di.hasintersection(dj, maxcover=cover, area=area[i], otherarea=area[j])) or di.hasintersection(dj, iou=iou, area=area[i], otherarea=area[j]))):  
                 suppressed.add(j)
-    return sorted([d for (j,d) in enumerate(detlist) if j not in suppressed], key=lambda x: x.confidence())  # smallest to biggest confidence for display layering
+    detlist_nms = [d for (j,d) in enumerate(detlist) if j not in suppressed]  # filter
+    detlist_nms.sort(key=lambda x: x.confidence())  # smallest to biggest confidence for display layering, in-place
+    return detlist_nms
 
 
 def greedy_assignment(srclist, dstlist, miniou=0.0, bycategory=False):
@@ -886,3 +897,5 @@ def greedy_assignment(srclist, dstlist, miniou=0.0, bycategory=False):
 
 def RandomDetection(W=640, H=480):
     return Detection(xmin=np.random.rand()*W, ymin=np.random.rand()*H, width=np.random.rand()*100, height=np.random.rand()*100, category=str(np.random.rand()), confidence=np.random.rand())
+
+
